package com.bergerkiller.bukkit.nolagg;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.block.BlockFace;
import org.bukkit.craftbukkit.CraftServer;
import org.bukkit.craftbukkit.CraftWorld;
import org.bukkit.craftbukkit.entity.CraftEntity;
import org.bukkit.craftbukkit.entity.CraftPlayer;
import org.bukkit.entity.ExperienceOrb;
import org.bukkit.entity.Item;
import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.Event;

import net.minecraft.server.Chunk;
import net.minecraft.server.ChunkProviderServer;
import net.minecraft.server.Entity;
import net.minecraft.server.EntityPlayer;
import net.minecraft.server.MathHelper;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.NibbleArray;
import net.minecraft.server.Packet;
import net.minecraft.server.World;
import net.minecraft.server.WorldServer;

public class Util {
	public static final int view = Bukkit.getServer().getViewDistance();
	public static final int viewWidth = view + view + 1;
	public static final int chunkarea = viewWidth * viewWidth;
	public static final int blockView = 32 + (view << 4);
		
	public static void broadcast(Chunk chunk, Packet packet) {
		broadcast(chunk.world, chunk.x, chunk.z, packet);
	}
	public static void broadcast(Chunk chunk, Collection<Packet> packets) {
		broadcast(chunk.world, chunk.x, chunk.z, packets);
	}
	public static void broadcast(Chunk chunk, Packet[] packets) {
		broadcast(chunk.world, chunk.x, chunk.z, packets);
	}
	public static void broadcast(World world, int cx, int cz, Packet packet) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					ep.netServerHandler.sendPacket(packet);
				}
			}
		}
	}
	public static void broadcast(World world, int cx, int cz, Collection<Packet> packets) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					for (Packet p : packets) {
						ep.netServerHandler.sendPacket(p);
					}
				}
			}
		}
	}
	public static void broadcast(World world, int cx, int cz, Packet[] packets) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					for (Packet p : packets) {
						ep.netServerHandler.sendPacket(p);
					}
				}
			}
		}
	}
	public static boolean isNear(Entity e, int cx, int cz) {
		return isNear((int) e.locX >> 4, (int) e.locZ >> 4, cx, cz);
	}
	public static boolean isNear(int cx1, int cz1, int cx2, int cz2) {
		cx1 -= cx2;
		if (cx1 > 15) return false;
		if (cx1 < -15) return false;
		cz1 -= cz2;
		if (cz1 > 15) return false;
		if (cz1 < -15) return false;
		return true;
	}
	public static boolean in(String item, String... items) {
		for (String i : items) {
			if (item.equalsIgnoreCase(i)) return true;
		}
		return false;
	}
	
	public static double useOld(double oldvalue, double newvalue) {
		return Util.useOld(oldvalue, newvalue, 0.2);
	}
	public static double useOld(double oldvalue, double newvalue, double peruseold) {
		return oldvalue + (peruseold * (newvalue - oldvalue));
	}
	
	public static List<WorldServer> getWorlds() {
		try {
			return ((CraftServer) Bukkit.getServer()).getHandle().server.worlds;
		} catch (NullPointerException ex) {
			return new ArrayList<WorldServer>();
		}
	}
    public static long toLong(int msw, int lsw) {
        return ((long) msw << 32) + lsw - Integer.MIN_VALUE;
    }
    public static boolean callEvent(Event event) {
    	Bukkit.getServer().getPluginManager().callEvent(event);
    	if (event instanceof Cancellable) {
    	    return !((Cancellable) event).isCancelled();
    	} else {
    		return true;
    	}
    }
	public static BlockFace getDirection(int yaw) {
        while (yaw <= -180) yaw += 360;
        while (yaw > 180) yaw -= 360;
		switch ((int) yaw) {
		case 0 : return BlockFace.NORTH;
		case 45 : return BlockFace.NORTH_EAST;
		case 90 : return BlockFace.EAST;
		case 135 : return BlockFace.SOUTH_EAST;
		case 180 : return BlockFace.SOUTH;
		case -135 : return BlockFace.SOUTH_WEST;
		case -90 : return BlockFace.WEST;
		case -45 : return BlockFace.NORTH_WEST;
		}
		//Let's apply angle differences
		if (yaw >= -22.5 && yaw < 22.5) {
			return BlockFace.NORTH;
		} else if (yaw >= 22.5 && yaw < 67.5) {
			return BlockFace.NORTH_EAST;
		} else if (yaw >= 67.5 && yaw < 112.5) {
			return BlockFace.EAST;
		} else if (yaw >= 112.5 && yaw < 157.5) {
			return BlockFace.SOUTH_EAST;
		} else if (yaw >= -67.5 && yaw < -22.5) {
			return BlockFace.NORTH_WEST;
		} else if (yaw >= -112.5 && yaw < -67.5) {
			return BlockFace.WEST;
		} else if (yaw >= -157.5 && yaw < -112.5) {
			return BlockFace.SOUTH_WEST;
		} else {
			return BlockFace.SOUTH;
		}
	}	
	public static int toChunk(double value) {
		return MathHelper.floor(value) >> 4;
	}
	
	public static boolean isPlayerNearBlock(Player player, final int bx, final int bz) {
		return isPlayerNearBlock(((CraftPlayer) player).getHandle(), bx, bz);
	}
	public static boolean isPlayerNearBlock(Entity player, final int bx, final int bz) {
		if (Math.abs(MathHelper.floor(player.locX) - bx) > blockView) return false;
		if (Math.abs(MathHelper.floor(player.locZ) - bz) > blockView) return false;
		return true;
	}
	public static void fillEntities(org.bukkit.World world, List<org.bukkit.entity.Entity> entitylist, List<Item> itemlist, List<ExperienceOrb> orblist) {
		fillEntities(((CraftWorld) world).getHandle(), entitylist, itemlist, orblist);
	}
	public static void fillEntities(World world, List<org.bukkit.entity.Entity> entitylist, List<Item> itemlist, List<ExperienceOrb> orblist) {
		CraftEntity e;
		for (Object obj : world.entityList) {
			e = (CraftEntity) ((net.minecraft.server.Entity) obj).getBukkitEntity();
			if (e.isDead()) continue;
			if (e instanceof Item) {
				if (ItemHandler.isIgnoredItem(e)) continue;
				if (itemlist != null) itemlist.add((Item) e);
			} else if (e instanceof ExperienceOrb) {
				if (orblist != null) orblist.add((ExperienceOrb) e);
			}
			if (entitylist != null) entitylist.add(e);
		}
	}

    public static double round(double Rval, int Rpl) {
    	double p = Math.pow(10, Rpl);
    	return Math.round(Rval * p) / p;
    }
    
	public static NibbleArray getNibble(final NibbleArray old, final byte[] newdata, final int bits) {
		if (old.a.length != newdata.length) {
			return new NibbleArray(newdata, bits);
		} else {
			return old;
		}
	}
	public static MinecraftServer getMCServer() {
		return ((CraftServer) Bukkit.getServer()).getServer();
	}
	public static org.bukkit.Chunk getNotLoadChunk(org.bukkit.World world, int x, int z) {
		Chunk chunk = getNotLoadChunk(ChunkHandler.getNative(world), x, z);
		return chunk == null ? null : chunk.bukkitChunk;
	}
	public static Chunk getNotLoadChunk(World world, int x, int z) {
		return getNotLoadChunk(((WorldServer) world).chunkProviderServer, x, z);
	}
	public static Chunk getNotLoadChunk(ChunkProviderServer chunkprovider, int x, int z) {
		return chunkprovider.chunks.get(x, z);
	}
	
	public static void send(Player player, Packet packet, boolean throughListeners) {
		if (player == null) return;
		send(((CraftPlayer) player).getHandle(), packet, throughListeners);
	}
	public static void send(EntityPlayer player, Packet packet, boolean throughListeners) {
		if (packet == null || player == null) return;
		if (player.netServerHandler == null || player.netServerHandler.disconnected) return;
		if (throughListeners) {
			player.netServerHandler.sendPacket(packet);
		} else if (player.netServerHandler.networkManager != null) {
			player.netServerHandler.networkManager.queue(packet);
		}
	}
}
